@@ -1,6 +1,6 @@
  /***************************************************************************\
 |*                                                                           *|
-|*       Copyright 1993-2014 NVIDIA, Corporation.  All rights reserved.      *|
+|*       Copyright 1993-2015 NVIDIA, Corporation.  All rights reserved.      *|
 |*                                                                           *|
 |*     NOTICE TO USER:   The source code  is copyrighted under  U.S. and     *|
 |*     international laws.  Users and possessors of this source code are     *|
@@ -11,7 +11,7 @@
 |*     tion and  internal comments to the code,  notices to the end user     *|
 |*     as follows:                                                           *|
 |*                                                                           *|
-|*       Copyright 1993-2014 NVIDIA, Corporation.  All rights reserved.      *|
+|*       Copyright 1993-2015 NVIDIA, Corporation.  All rights reserved.      *|
 |*                                                                           *|
 |*     NVIDIA, CORPORATION MAKES NO REPRESENTATION ABOUT THE SUITABILITY     *|
 |*     OF  THIS SOURCE  CODE  FOR ANY PURPOSE.  IT IS  PROVIDED  "AS IS"     *|
@@ -100,7 +100,7 @@ typedef NvU8*              NVREGSTR;
 #if (defined(NV_UNIX) || defined(vxworks) || defined(NV_WINDOWS_CE) ||  \
      defined(__arm) || defined(__IAR_SYSTEMS_ICC__) || defined(NV_QNX) || \
      defined(NV_INTEGRITY) || defined(NV_HOS) || defined(NV_MODS) || \
-     defined(__GNUC__) || defined(NV_MACINTOSH_64)) && \
+     defined(__GNUC__) || defined(__clang__) || defined(NV_MACINTOSH_64)) && \
     (!defined(NV_MACINTOSH) || defined(NV_MACINTOSH_64))
 typedef unsigned int       NvV32; /* "void": enumerated or multiple fields   */
 typedef unsigned int       NvU32; /* 0 to 4294967295                         */
@@ -117,9 +117,9 @@ typedef signed int         NvS32; /* -2147483648 to 2147483647               */
 #endif
 
 /* 64-bit types for compilers that support them, plus some obsolete variants */
-#if defined(__GNUC__) || defined(__arm) || defined(__IAR_SYSTEMS_ICC__) || \
-    defined(__ghs__) || defined(_WIN64) || defined(__SUNPRO_C) || \
-    defined(__SUNPRO_CC) || defined (__xlC__)
+#if defined(__GNUC__) || defined(__clang__) || defined(__arm) || \
+    defined(__IAR_SYSTEMS_ICC__) || defined(__ghs__) || defined(_WIN64) || \
+    defined(__SUNPRO_C) || defined(__SUNPRO_CC) || defined (__xlC__)
 typedef unsigned long long NvU64; /* 0 to 18446744073709551615          */
 typedef          long long NvS64; /* 2^-63 to 2^63-1                    */
 #else
@@ -279,7 +279,7 @@ typedef unsigned char *PUCHAR;
 #endif
 
 /* Aligns fields in structs  so they match up between 32 and 64 bit builds */
-#if defined(__GNUC__) || defined(NV_QNX) || defined(NV_HOS)
+#if defined(__GNUC__) || defined(__clang__) || defined(NV_QNX) || defined(NV_HOS)
 #define NV_ALIGN_BYTES(size) __attribute__ ((aligned (size)))
 #elif defined(__arm)
 #define NV_ALIGN_BYTES(size) __align(ALIGN)
@@ -293,16 +293,16 @@ typedef unsigned char *PUCHAR;
 // This macro form accounts for the fact that __declspec on Windows is required
 // before the variable type,
 // and NV_ALIGN_BYTES is required after the variable name.
-#if defined(NV_WINDOWS)
-#define NV_DECLARE_ALIGNED(TYPE_VAR, ALIGN) __declspec(align(ALIGN)) TYPE_VAR
-#elif defined(__GNUC__) || defined(NV_QNX) || defined(NV_HOS)
+#if defined(__GNUC__) || defined(__clang__) || defined(NV_QNX) || defined(NV_HOS)
 #define NV_DECLARE_ALIGNED(TYPE_VAR, ALIGN) TYPE_VAR __attribute__ ((aligned (ALIGN)))
+#elif defined(_MSC_VER)
+#define NV_DECLARE_ALIGNED(TYPE_VAR, ALIGN) __declspec(align(ALIGN)) TYPE_VAR
 #elif defined(__arm)
 #define NV_DECLARE_ALIGNED(TYPE_VAR, ALIGN) __align(ALIGN) TYPE_VAR
 #endif
 
 // NVRM_IMPORT is defined on windows for nvrm4x build (nvrm4x.lib).
-#if (defined(NV_WINDOWS) && defined(NVRM4X_BUILD))
+#if (defined(_MSC_VER) && defined(NVRM4X_BUILD))
 #define NVRM_IMPORT __declspec(dllimport)
 #else
 #define NVRM_IMPORT
@@ -323,7 +323,7 @@ typedef unsigned char *PUCHAR;
 // place to re-locate these from nvos.h which cannot be included by a number
 // of builds that need them
 
-#if defined(NV_WINDOWS)
+#if defined(_MSC_VER)
 
     #define NV_INLINE __inline
 
@@ -340,9 +340,9 @@ typedef unsigned char *PUCHAR;
 
     #define NV_FORCERESULTCHECK
 
-#else // ! defined(NV_WINDOWS)
+#else // ! defined(_MSC_VER)
 
-    #if defined(__GNUC__) || defined(__INTEL_COMPILER)
+    #if defined(__GNUC__) || defined(__clang__) || defined(__INTEL_COMPILER)
     #define NV_INLINE __inline__
     #elif defined (macintosh) || defined(__SUNPRO_C) || defined(__SUNPRO_CC)
     #define NV_INLINE inline
@@ -361,6 +361,12 @@ typedef unsigned char *PUCHAR;
             #else
             #define NV_FORCEINLINE __inline__
             #endif
+        #elif defined(__clang__)
+            #if __has_attribute(always_inline)
+            #define NV_FORCEINLINE __attribute__((always_inline)) __inline__
+            #else
+            #define NV_FORCEINLINE __inline__
+            #endif
         #elif defined(__arm) && (__ARMCC_VERSION >= 220000)
             // RVDS 2.2 also supports forceinline, but ADS 1.2 does not
             #define NV_FORCEINLINE __forceinline
@@ -389,11 +395,17 @@ typedef unsigned char *PUCHAR;
         #else
         #define NV_FORCERESULTCHECK
         #endif
+    #elif defined(__clang__)
+        #if __has_attribute(warn_unused_result)
+        #define NV_FORCERESULTCHECK __attribute__((warn_unused_result))
+        #else
+        #define NV_FORCERESULTCHECK
+        #endif
     #else /* defined(__GNUC__) */
         #define NV_FORCERESULTCHECK
     #endif
 
-#endif  // defined(NV_WINDOWS)
+#endif  // defined(_MSC_VER)
 
 /*!
  * Fixed-point master data types.
@@ -540,9 +552,13 @@ typedef NvU64                                                      NvUFXP48_16;
         !!DRF_VAL(_TYPES, _FXP, _FRACTIONAL_MSB((x), (y)),                    \
             ((NvUFXP##x##_##y) (fxp))))
 
+//
 // 32-bit Signed FXP:
+// Some compilers do not support left shift negative values
+// so typecast integer to NvU32 instead of NvS32 
+//
 #define NV_TYPES_S32_TO_SFXP_X_Y(x, y, integer)                               \
-    ((NvSFXP##x##_##y) (((NvS32) (integer)) <<                                \
+    ((NvSFXP##x##_##y) (((NvU32) (integer)) <<                                \
                         DRF_SHIFT(NV_TYPES_FXP_INTEGER((x), (y)))))
 
 #define NV_TYPES_S32_TO_SFXP_X_Y_SCALED(x, y, integer, scale)                 \
