@@ -656,9 +656,9 @@ void NVKMS_API_CALL nvkms_close_gpu(NvU32 gpuId)
     __rm_ops.close_gpu(gpuId, nvkms_nvidia_stack);
 }
 
-NvBool NVKMS_API_CALL nvkms_list_gpus(NvU32 *gpu_ids, NvU32 *gpu_count)
+NvU32 NVKMS_API_CALL nvkms_enumerate_gpus(nv_gpu_info_t *gpu_info)
 {
-    return NV_FALSE;
+    return 0;
 }
 
 /*************************************************************************
@@ -686,6 +686,28 @@ NvBool NVKMS_API_CALL nvkms_ioctl_from_kapi
     return NV_FALSE;
 }
 
+
+/*************************************************************************
+ * APIs for locking.
+ *************************************************************************/
+
+nvkms_sema_handle_t* nvkms_sema_alloc(void)
+{
+    return NULL;
+}
+
+void nvkms_sema_free(nvkms_sema_handle_t *sema)
+{
+}
+
+void nvkms_sema_down(nvkms_sema_handle_t *seam)
+{
+}
+
+void nvkms_sema_up(nvkms_sema_handle_t *sema)
+{
+}
+
 /*************************************************************************
  * File operation callback functions.
  *************************************************************************/
