@@ -217,8 +217,8 @@ void NVKMS_API_CALL nvkms_close_gpu(NvU32 gpuId);
 /*!
  * Enumerate nvidia gpus.
  */
-NvBool NVKMS_API_CALL nvkms_list_gpus(NvU32 *gpu_ids, NvU32 *gpu_count);
 
+NvU32 NVKMS_API_CALL nvkms_enumerate_gpus(nv_gpu_info_t *gpu_info);
 
 /*!
  * NVKMS interface for kernel space NVKMS clients like KAPI
@@ -239,5 +239,16 @@ NvBool NVKMS_API_CALL nvkms_ioctl_from_kapi
     NvU32 cmd, void *params_address, const size_t params_size
 );
 
+/*!
+ * APIs for locking.
+ */
+
+typedef struct nvkms_sema_t nvkms_sema_handle_t;
+
+nvkms_sema_handle_t*    nvkms_sema_alloc    (void);
+void                    nvkms_sema_free     (nvkms_sema_handle_t *sema);
+void                    nvkms_sema_down     (nvkms_sema_handle_t *sema);
+void                    nvkms_sema_up       (nvkms_sema_handle_t *sema);
+
 #endif /* _NVIDIA_MODESET_OS_INTERFACE_H_ */
 
