@@ -12,6 +12,7 @@
 
 #include "nvtypes.h"
 
+#include "nv-gpu-info.h"
 #include "nvkms-api-types.h"
 
 #define __NVKMS_KAPI_H__
@@ -167,7 +168,6 @@ struct NvKmsKapiDisplayInfo {
 };
 
 struct NvKmsKapiPlaneConfig {
-
     struct NvKmsKapiSurface *surface;
 
     NvU16 srcX, srcY;
@@ -175,24 +175,41 @@ struct NvKmsKapiPlaneConfig {
 
     NvU16 dstX, dstY;
     NvU16 dstWidth, dstHeight;
-
 };
 
-struct NvKmsKapiModeSetConfig {
-
-    NvU32 headsMask;
+struct NvKmsKapiPlaneRequestedConfig {
+    struct NvKmsKapiPlaneConfig config;
     struct {
+        NvBool surfaceChanged : 1;
+        NvBool srcXYChanged   : 1;
+        NvBool srcWHChanged   : 1;
+        NvBool dstXYChanged   : 1;
+        NvBool dstWHChanged   : 1;
+    } flags;
+};
 
-        NvU32      numDisplays;
-        NvKmsKapiDisplay displays[NVKMS_KAPI_MAX_CLONE_DISPLAYS];
+struct NvKmsKapiHeadModeSetConfig {
+    NvU32  numDisplays;
+    NvKmsKapiDisplay displays[NVKMS_KAPI_MAX_CLONE_DISPLAYS];
 
-        struct NvKmsKapiDisplayMode mode;
+    struct NvKmsKapiDisplayMode mode;
+};
 
-        NvU32 planesMask;
-        struct NvKmsKapiPlaneConfig planes[NVKMS_KAPI_PLANE_MAX];
+struct NvKmsKapiHeadRequestedConfig {
+    struct NvKmsKapiHeadModeSetConfig modeSetConfig;
+    struct {
+        NvBool displaysChanged : 1;
+        NvBool modeChanged     : 1;
+    } flags;
 
-    } heads[NVKMS_KAPI_MAX_HEADS];
+    struct NvKmsKapiPlaneRequestedConfig
+        planeRequestedConfig[NVKMS_KAPI_PLANE_MAX];
+};
 
+struct NvKmsKapiRequestedModeSetConfig {
+    NvU32 headsMask;
+    struct NvKmsKapiHeadRequestedConfig
+        headRequestedConfig[NVKMS_KAPI_MAX_HEADS];
 };
 
 struct NvKmsKapiEventDisplayChanged {
@@ -203,6 +220,11 @@ struct NvKmsKapiEventDynamicDisplayConnected {
     NvKmsKapiDisplay display;
 };
 
+struct NvKmsKapiEventFlipOccurred {
+    NvU32 head;
+    NvKmsKapiPlaneType plane;
+};
+
 struct NvKmsKapiEvent {
     enum NvKmsEventType type;
 
@@ -213,13 +235,17 @@ struct NvKmsKapiEvent {
     union {
         struct NvKmsKapiEventDisplayChanged displayChanged;
         struct NvKmsKapiEventDynamicDisplayConnected dynamicDisplayConnected;
+        struct NvKmsKapiEventFlipOccurred flipOccurred;
     } u;
 };
 
 struct NvKmsKapiAllocateDeviceParams {
+    /* [IN] GPU ID obtained from enumerateGpus() */
     NvU32 gpuId;
 
+    /* [IN] Private data of device allocator */
     void *privateData;
+    /* [IN] Event callback */
     void (*eventCallback)(const struct NvKmsKapiEvent *event);
 };
 
@@ -233,25 +259,18 @@ struct NvKmsKapiFunctionsTable {
     /*!
      * Enumerate the available physical GPUs that can be used with NVKMS.
      *
-     * \param [out]     gpuIds    The IDs of the enumerated GPUs. If gpuIds is
-     *                            NULL, the total number of GPUs in the system is
-     *                            returned in gpuCount and no IDs are returned.
+     * \param [out]  gpuInfo  The information of the enumerated GPUs.
+     *                        It is an array of NVIDIA_MAX_GPUS elements.
      *
-     * \param [in/out]  gpuCount  Set to the size of gpuIds array when calling.
-     *                            Returns the number of gpuIds that were placed
-     *                            in gpuIds.
-     *
-     * \return  NV_TRUE on success. NV_FALSE if gpuCount is too low.
+     * \return  Count of enumerated gpus.
      */
-    NvBool NVKMS_KAPI_CALL (*enumerateGpus)(NvU32 *gpuIds, NvU32 *gpuCount);
+    NvU32 NVKMS_KAPI_CALL (*enumerateGpus)(nv_gpu_info_t *gpuInfo);
 
     /*!
      * Allocate an NVK device using which you can query/allocate resources on
      * GPU and do modeset.
      *
-     * \param [in]  params  Contains a GPU ID obtained from enumerateGpus(),
-     *                      private data allocator wants to associate with
-     *                      devce, and event callback.
+     * \param [in]  params  Parameters required for device allocation.
      *
      * \return  An valid device handle on success, NULL on failure.
      */
@@ -284,6 +303,19 @@ struct NvKmsKapiFunctionsTable {
      */
     void NVKMS_KAPI_CALL (*releaseOwnership)(struct NvKmsKapiDevice *device);
 
+    /*!
+     * Migrate ownership from the NvKmsKapiDevice to the NVKMS device
+     * specified by the (nvKmsFd, nvKmsDeviceHandle) pair.
+     *
+     * \param [in]  device   A device returned by allocateDevice().
+     * \param [in]  nvKmsFd  File descriptor of a user-space NVKMS client.
+     * \param [in]  nvKmsDeviceHandle  The device handle in nvKmsFd's namespace.
+     *
+     * \return NV_TRUE on success, NV_FALSE on failure.
+     */
+    NvBool NVKMS_KAPI_CALL (*migrateOwnership)(struct NvKmsKapiDevice *device,
+                                               const NvU32 nvKmsFd,
+                                               const NvU32 nvKmsDeviceHandle);
     /*!
      * Registers for notification, via
      * NvKmsKapiAllocateDeviceParams::eventCallback, of the events specified
@@ -408,6 +440,35 @@ struct NvKmsKapiFunctionsTable {
         struct NvKmsKapiDevice *device, NvU64 size
     );
 
+    /*!
+     * Import some unformatted memory of the specified size.
+     *
+     * This function accepts a driver-specific parameter structure representing
+     * memory allocated elsewhere and imports it to a NVKMS KAPI memory object
+     * of the specified size.
+     *
+     * \param [in] device  A device allocated using allocateDevice().  The
+     *                     memory being imported must have been allocated
+     *                     against the same physical device this device object
+     *                     represents.
+     *
+     * \param [in] size    Size, in bytes, of the memory being imported.
+     *
+     * \param [in] nvKmsParamsUser Userspace pointer to driver-specific
+     *                             parameters describing the memory object being
+     *                             imported.
+     *
+     * \param [in] nvKmsParamsSize Size of the driver-specific parameter struct.
+     *
+     * \return A valid memory handle on success, NULL on failure.
+     */
+    struct NvKmsKapiMemory* NVKMS_KAPI_CALL (*importMemory)
+    (
+        struct NvKmsKapiDevice *device, NvU64 size,
+        NvU64 nvKmsParamsUser,
+        NvU64 nvKmsParamsSize
+    );
+
     /*!
      * Free memory allocated using allocateMemory()
      *
@@ -417,7 +478,7 @@ struct NvKmsKapiFunctionsTable {
      *
      * \return NV_TRUE on success, NV_FALSE if memory is in use.
      */
-    NvBool NVKMS_KAPI_CALL (*freeMemory)
+    void NVKMS_KAPI_CALL (*freeMemory)
     (
         struct NvKmsKapiDevice *device, struct NvKmsKapiMemory *memory
     );
@@ -473,13 +534,20 @@ struct NvKmsKapiFunctionsTable {
      *
      * \param [in]  pitch   Byte pitch of the surface.
      *
+     * \param [in]  nvkmsParamsUser Userspace pointer to driver-specific surface
+     *                              properties.
+     *
+     * \param [in]  nvkmsParamsSize Size of the memory pointed to by
+     *                              nvkmsParamsUser.
+     *
      * \return  An valid surface handle on success.  NULL on failure.
      */
     struct NvKmsKapiSurface* NVKMS_KAPI_CALL (*createSurface)
     (
         struct NvKmsKapiDevice *device,
         struct NvKmsKapiMemory *memory, enum NvKmsSurfaceMemoryFormat format,
-        NvU32 width, NvU32 height, NvU32 pitch
+        NvU32 width, NvU32 height, NvU32 pitch,
+        NvU64 nvKmsParamsUser, NvU64 nvKmsParamsSize
     );
 
     /*!
@@ -541,22 +609,47 @@ struct NvKmsKapiFunctionsTable {
     /*!
      * Apply a mode configuration to the device.
      *
-     * \param [in]  device   A device allocated using allocateDevice().
+     * Client can describe damaged part of configuration but still it is must
+     * to describe entire configuration.
      *
-     * \param [in]  config   Parameters describing a device-wide display
-     *                       configuration.
+     * \param [in]  device            A device allocated using allocateDevice().
      *
-     * \param [in]  commit   If set to 0 them call will only validate mode
-     *                       configuration, will not apply it.
+     * \param [in]  requestedConfig   Parameters describing a device-wide
+     *                                display configuration.
+     *
+     * \param [in]  commit            If set to 0 them call will only validate
+     *                                mode configuration, will not apply it.
      *
      * \return NV_TRUE on success, NV_FALSE on failure.
      */
     NvBool NVKMS_KAPI_CALL (*applyModeSetConfig)
     (
         struct NvKmsKapiDevice *device,
-        const struct NvKmsKapiModeSetConfig *config, const NvBool commit
+        const struct NvKmsKapiRequestedModeSetConfig *requestedConfig,
+        const NvBool commit
     );
 
+    /*!
+     * Return status of flip.
+     *
+     * \param  [in]  device   A device allocated using allocateDevice().
+     *
+     * \param  [in]  head     A head returned by getDeviceResourcesInfo().
+     *
+     * \param  [in]  plane    A plane type.
+     *
+     * \param  [out] pending  Return TRUE if head has pending flip for
+     *                        given plane.
+     *
+     * \return NV_TRUE on success, NV_FALSE on failure.
+     */
+    NvBool NVKMS_KAPI_CALL (*getFlipPendingStatus)
+    (
+        const struct NvKmsKapiDevice *device,
+        const NvU32 head,
+        const NvKmsKapiPlaneType plane,
+        NvBool *pending
+    );
 };
 
 /** @} */
