@@ -410,6 +410,32 @@
 #define __NV_MEMORY_POOL_SIZE MemoryPoolSize
 #define NV_REG_MEMORY_POOL_SIZE NV_REG_STRING(__NV_MEMORY_POOL_SIZE)
 
+/*
+ * Option: TCEBypassMode
+ *
+ * Description:
+ *
+ * When this option is enabled, the NVIDIA kernel module will attempt to setup
+ * all GPUs in "TCE bypass mode", in which DMA mappings of system memory bypass
+ * the IOMMU/TCE remapping hardware on IBM POWER systems. This is typically
+ * necessary for CUDA applications in which large system memory mappings may
+ * exceed the default TCE remapping capacity when operated in non-bypass mode.
+ *
+ * This option has no effect on non-POWER platforms.
+ *
+ * Possible Values:
+ * 
+ *  0: system default TCE mode on all GPUs
+ *  1: enable TCE bypass mode on all GPUs
+ *  2: disable TCE bypass mode on all GPUs
+ */
+#define __NV_TCE_BYPASS_MODE TCEBypassMode
+#define NV_REG_TCE_BYPASS_MODE NV_REG_STRING(__NV_TCE_BYPASS_MODE)
+
+#define NV_TCE_BYPASS_MODE_DEFAULT  0
+#define NV_TCE_BYPASS_MODE_ENABLE   1
+#define NV_TCE_BYPASS_MODE_DISABLE  2
+
 #if defined(NV_DEFINE_REGISTRY_KEY_TABLE)
 
 /*
@@ -431,6 +457,7 @@ NV_DEFINE_REG_ENTRY(__NV_REGISTER_FOR_ACPI_EVENTS, 1);
 NV_DEFINE_REG_ENTRY(__NV_CHECK_PCI_CONFIG_SPACE, NV_CHECK_PCI_CONFIG_SPACE_INIT);
 NV_DEFINE_REG_ENTRY(__NV_ENABLE_PCIE_GEN3, 0);
 NV_DEFINE_REG_ENTRY(__NV_ENABLE_MSI, 1);
+NV_DEFINE_REG_ENTRY(__NV_TCE_BYPASS_MODE, NV_TCE_BYPASS_MODE_DEFAULT);
 NV_DEFINE_REG_ENTRY_GLOBAL(__NV_MEMORY_POOL_SIZE, 0);
 
 NV_DEFINE_REG_STRING_ENTRY(__NV_REGISTRY_DWORDS, NULL);
@@ -468,6 +495,7 @@ nv_parm_t nv_parms[] = {
     NV_DEFINE_PARAMS_TABLE_ENTRY(__NV_CHECK_PCI_CONFIG_SPACE),
     NV_DEFINE_PARAMS_TABLE_ENTRY(__NV_ENABLE_PCIE_GEN3),
     NV_DEFINE_PARAMS_TABLE_ENTRY(__NV_MEMORY_POOL_SIZE),
+    NV_DEFINE_PARAMS_TABLE_ENTRY(__NV_TCE_BYPASS_MODE),
     {NULL, NULL, NULL}
 };
 
