@@ -325,6 +325,7 @@ typedef struct
     NvU16 subsystem_id;
     NvU16 subsystem_vendor;
     NvU32 gpu_id;
+    NvU32 iovaspace_id;
     struct
     {
         NvBool         bValid;
@@ -360,33 +361,38 @@ typedef struct
 } nv_state_t;
 
 // Forward define the gpu ops structures
-typedef struct gpuSession           *nvgpuSessionHandle_t;
-typedef struct gpuAddressSpace      *nvgpuAddressSpaceHandle_t;
-typedef struct gpuChannel           *nvgpuChannelHandle_t;
-typedef struct gpuObject            *nvgpuObjectHandle_t;
-typedef struct UvmGpuChannelPointers_tag *nvgpuChannelInfo_t;
-typedef struct UvmGpuCaps_tag       *nvgpuCaps_t;
-typedef struct gpuAllocInfo         *nvgpuAllocInfo_t;
-typedef struct gpuSurfaceMappingInfo *nvgpuSurfaceMapInfo_t;
-typedef struct UvmGpuP2PCapsParams_tag *nvgpuP2PCapsParams_t;
-typedef struct gpuVaAllocInfo       *nvgpuVaAllocInfo_t;
-typedef struct gpuMapInfo           *nvgpuMapInfo_t;
-typedef struct gpuFbInfo            *nvgpuFbInfo_t;
-typedef struct gpuFaultInfo         *nvgpuFaultInfo_t;
-typedef struct gpuAccessCntrInfo    *nvgpuAccessCntrInfo_t;
-typedef struct UvmGpuInfo_tag       *nvgpuInfo_t;
-typedef struct gpuPageLevelInfo     *nvgpuPageLevelInfo_t;
-typedef struct gpuChannelPhysInfo   *nvgpuChannelPhysInfo_t;
-typedef struct gpuChannelCtxBufferInfo *nvgpuChannelCtxBufferInfo_t;
-typedef struct gpuChannelBufferVa   *nvgpuChannelBufferVa_t;
-typedef struct gpuPmaAllocationOptions *nvgpuPmaAllocationOptions_t;
+typedef struct gpuSession                           *nvgpuSessionHandle_t;
+typedef struct gpuAddressSpace                      *nvgpuAddressSpaceHandle_t;
+typedef struct gpuChannel                           *nvgpuChannelHandle_t;
+typedef struct gpuObject                            *nvgpuObjectHandle_t;
+typedef struct UvmGpuChannelPointers_tag            *nvgpuChannelInfo_t;
+typedef struct UvmGpuCaps_tag                       *nvgpuCaps_t;
+typedef struct UvmGpuAllocInfo_tag                  *nvgpuAllocInfo_t;
+typedef struct gpuSurfaceMappingInfo                *nvgpuSurfaceMapInfo_t;
+typedef struct UvmGpuP2PCapsParams_tag              *nvgpuP2PCapsParams_t;
+typedef struct gpuVaAllocInfo                       *nvgpuVaAllocInfo_t;
+typedef struct gpuMapInfo                           *nvgpuMapInfo_t;
+typedef struct gpuFbInfo                            *nvgpuFbInfo_t;
+typedef struct UvmGpuFaultInfo_tag                  *nvgpuFaultInfo_t;
+typedef struct UvmGpuAccessCntrInfo_tag             *nvgpuAccessCntrInfo_t;
+typedef struct UvmGpuInfo_tag                       *nvgpuInfo_t;
+typedef struct gpuPageLevelInfo                     *nvgpuPageLevelInfo_t;
+typedef struct gpuChannelPhysInfo                   *nvgpuChannelPhysInfo_t;
+typedef struct gpuChannelCtxBufferInfo              *nvgpuChannelCtxBufferInfo_t;
+typedef struct gpuChannelBufferVa                   *nvgpuChannelBufferVa_t;
+typedef struct gpuPmaAllocationOptions              *nvgpuPmaAllocationOptions_t;
+typedef struct UvmGpuMemoryInfo_tag                 *nvgpuMemoryInfo_t;
+typedef struct UvmGpuExternalMappingInfo_tag        *nvgpuExternalMappingInfo_t;
+typedef struct UvmGpuChannelResourceInfo_tag        *nvgpuChannelResourceInfo_t;
+typedef struct UvmGpuChannelInstanceInfo_tag        *nvgpuChannelInstanceInfo_t;
+typedef struct UvmGpuChannelResourceBindParams_tag  *nvgpuChannelResourceBindParams_t;
 
 /*
  * flags
  */
 
 #define NV_FLAG_OPEN                   0x0001
-#define NV_FLAG_WAS_POSTED             0x0002
+// EMPTY SLOT                          0x0002
 #define NV_FLAG_CONTROL                0x0004
 #define NV_FLAG_MAP_REGS_EARLY         0x0008
 #define NV_FLAG_USE_BAR0_CFG           0x0010
@@ -400,6 +406,7 @@ typedef struct gpuPmaAllocationOptions *nvgpuPmaAllocationOptions_t;
 #define NV_FLAG_IN_RECOVERY            0x1000
 #define NV_FLAG_SKIP_CFG_CHECK         0x2000
 #define NV_FLAG_UNBIND_LOCK            0x4000
+#define NV_FLAG_ACQ_GPU_LOCK           0x8000
 
 #define NV_PM_ACPI_HIBERNATE    0x0001
 #define NV_PM_ACPI_STANDBY      0x0002
@@ -622,7 +629,6 @@ NV_STATUS  NV_API_CALL  nv_acpi_dod_method       (nv_state_t *, NvU32 *, NvU32 *
 NV_STATUS  NV_API_CALL  nv_acpi_rom_method       (nv_state_t *, NvU32 *, NvU32 *);
 NV_STATUS  NV_API_CALL  nv_log_error             (nv_state_t *, NvU32, const char *, va_list);
 
-NV_STATUS  NV_API_CALL  nv_alloc_os_descriptor_handle (nv_state_t *, NvS32, void *, NvU64, NvU32 *);
 NvU64      NV_API_CALL  nv_get_dma_start_address (nv_state_t *);
 NV_STATUS  NV_API_CALL  nv_set_primary_vga_status(nv_state_t *);
 NV_STATUS  NV_API_CALL  nv_pci_trigger_recovery  (nv_state_t *);
@@ -728,6 +734,7 @@ NV_STATUS  NV_API_CALL  rm_gpu_ops_channel_allocate(nvidia_stack_t *, nvgpuAddre
 NV_STATUS  NV_API_CALL  rm_gpu_ops_channel_destroy(nvidia_stack_t *, nvgpuChannelHandle_t);
 const char*  NV_API_CALL  rm_gpu_ops_channel_translate_error(nvidia_stack_t *, NvU32);
 NV_STATUS  NV_API_CALL  rm_gpu_ops_copy_engine_allocate(nvidia_stack_t *, nvgpuChannelHandle_t, NvU32, NvU32*, nvgpuObjectHandle_t *);
+NV_STATUS  NV_API_CALL  rm_gpu_ops_copy_engine_alloc(nvidia_stack_t *, nvgpuChannelHandle_t, NvU32, nvgpuObjectHandle_t *);
 NV_STATUS  NV_API_CALL rm_gpu_ops_memory_free(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvU64);
 NV_STATUS  NV_API_CALL rm_gpu_ops_memory_free_pa(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvU64);
 NV_STATUS  NV_API_CALL rm_gpu_ops_query_caps(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, nvgpuCaps_t);
@@ -740,10 +747,9 @@ NV_STATUS  NV_API_CALL rm_gpu_ops_kill_channel(nvidia_stack_t *, nvgpuChannelHan
 NV_STATUS  NV_API_CALL rm_gpu_ops_get_gmmu_fmt(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, void **);
 NV_STATUS  NV_API_CALL rm_gpu_ops_dup_allocation(nvidia_stack_t *, NvHandle, nvgpuAddressSpaceHandle_t, NvU64, nvgpuAddressSpaceHandle_t, NvU64*, NvBool);
 
-NV_STATUS  NV_API_CALL  rm_gpu_ops_dup_memory (nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvHandle, NvHandle hClientMem, NvHandle *hDupMemory);
+NV_STATUS  NV_API_CALL  rm_gpu_ops_dup_memory (nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvHandle, NvHandle, NvHandle *, nvgpuMemoryInfo_t);
 
 NV_STATUS  NV_API_CALL rm_gpu_ops_free_duped_handle(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvHandle);
-NV_STATUS  NV_API_CALL rm_gpu_ops_check_pseudo_p9(nvidia_stack_t *sp, nvgpuAddressSpaceHandle_t vaspace, NvBool *isPseudoP9);
 NV_STATUS  NV_API_CALL rm_gpu_ops_get_fb_info(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, nvgpuFbInfo_t);
 NV_STATUS  NV_API_CALL rm_gpu_ops_get_gpu_ids(nvidia_stack_t *sp, NvU8 *pUuid, unsigned uuidLength, NvU32 *pDeviceId, NvU32 *pSubdeviceId);
 NV_STATUS NV_API_CALL rm_gpu_ops_own_page_fault_intr(nvidia_stack_t *, NvU8 *, unsigned, NvBool);
@@ -757,7 +763,6 @@ void       NV_API_CALL rm_gpu_ops_free_mem_handles(nvidia_stack_t *, void**, NvU
 NV_STATUS  NV_API_CALL  rm_gpu_ops_set_page_directory (nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvU64, unsigned, NvBool);
 NV_STATUS  NV_API_CALL  rm_gpu_ops_unset_page_directory (nvidia_stack_t *, nvgpuAddressSpaceHandle_t);
 NV_STATUS  NV_API_CALL rm_gpu_ops_bind_channel(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvHandle, NvHandle, NvU32, nvgpuChannelBufferVa_t);
-NV_STATUS  NV_API_CALL rm_gpu_ops_stop_channel(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvHandle, NvHandle, NvP64);
 NV_STATUS  NV_API_CALL rm_gpu_ops_get_ctx_buffer_count(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvU32 *);
 NV_STATUS  NV_API_CALL rm_gpu_ops_get_ctx_buffer_info(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvHandle, NvHandle, NvU32, nvgpuChannelCtxBufferInfo_t);
 NV_STATUS  NV_API_CALL rm_gpu_ops_get_ctx_buffer_phys_info(nvidia_stack_t *, void *, NvU64, NvU64 *);
@@ -766,8 +771,16 @@ NV_STATUS  NV_API_CALL rm_gpu_ops_stop_vaspace_channels(nvidia_stack_t *, nvgpuS
 NV_STATUS  NV_API_CALL rm_gpu_ops_p2p_object_create(nvidia_stack_t *, nvgpuSessionHandle_t, NvProcessorUuid *, NvProcessorUuid *, NvHandle *);
 void       NV_API_CALL rm_gpu_ops_p2p_object_destroy(nvidia_stack_t *, nvgpuSessionHandle_t, NvHandle);
 NV_STATUS  NV_API_CALL rm_gpu_ops_get_big_page_size(nvidia_stack_t*, nvgpuAddressSpaceHandle_t, NvU32 *);
-
+NV_STATUS  NV_API_CALL rm_gpu_ops_get_external_alloc_ptes(nvidia_stack_t*, nvgpuAddressSpaceHandle_t, NvHandle, NvU64, NvU64, nvgpuExternalMappingInfo_t);
+NV_STATUS  NV_API_CALL rm_gpu_ops_retain_channel(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvHandle, NvHandle, nvgpuChannelInstanceInfo_t);
+NV_STATUS  NV_API_CALL rm_gpu_ops_retain_channel_resources(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvP64, NvU32, nvgpuChannelResourceInfo_t);
+NV_STATUS  NV_API_CALL rm_gpu_ops_bind_channel_resources(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvP64, NvU32, nvgpuChannelResourceBindParams_t);
+void       NV_API_CALL rm_gpu_ops_release_channel(nvidia_stack_t *, NvP64);
+void       NV_API_CALL rm_gpu_ops_release_channel_resources(nvidia_stack_t *, NvP64*, NvU32);
+void       NV_API_CALL rm_gpu_ops_stop_channel(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvP64);
+NV_STATUS  NV_API_CALL rm_gpu_ops_get_channel_resource_ptes(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvP64, NvU64, NvU64, nvgpuExternalMappingInfo_t);
 void       NV_API_CALL rm_kernel_rmapi_op(nvidia_stack_t *sp, void *ops_cmd);
+NV_STATUS  NV_API_CALL rm_gpu_copy_mmu_faults(nvidia_stack_t *, nv_state_t *, NvU32 *);
 
 #endif /* NVWATCH */
 
